@@ -4,7 +4,7 @@ from bruhanimate.bruheffects    import BaseEffect, StaticEffect, OffsetEffect, N
 from bruhanimate.bruhffer       import Buffer
 from bruhanimate.bruhrenderer   import BaseRenderer, EffectRenderer, CenterRenderer, PanRenderer
 from bruhanimate import images
-from bruhanimate import demo as demo
+from bruhanimate import demo
 
 __all__ = [
     "Screen",
